# use-mst-immer

A mobx-state-tree powered [use-immer](https://github.com/immerjs/use-immer).

# Installation

`npm install mobx mobx-state-tree use-mst-immer`

# Usage

## useMstImmer

```javascript
import React from 'react';
import { useMstImmer } from 'use-mst-immer';

function App() {
    const [person, updatePerson] = useMstImmer({
        name: 'Kim',
        age: 37,
        dogs: types.array(types.string),
    });

    function updateName(name) {
        updatePerson((state) => {
            state.name = name;
        });
    }

    function becomeOlder() {
        updatePerson((state) => {
            state.age++;
        });
    }

    return (
        <div className="App">
            <h1>
                Hello {person.name} ({person.age})
            </h1>
            <input
                onChange={(e) => {
                    updateName(e.target.value);
                }}
                value={person.name}
            />
            <br />
            <button onClick={becomeOlder}>Older</button>
            <input placeholder={'Add dog'} onKeyDown={(ev => {

            })}>
        </div>
    );
}
```

### Managing state as simple useState hook

When passing a value to the updater instead of a function, `useImmer` hook behaves the same as useState hook and updates the state with that value.

```javascript
import React from 'react';
import { useImmer } from 'use-immer';

function BirthDayCelebrator() {
    const [age, setAge] = useImmer(20);

    function birthDay(event) {
        setAge(age + 1);
        alert(`Happy birthday #${age} Anon! hope you good`);
    }

    return (
        <div>
            <button onClick={birthDay}>It is my birthday</button>
        </div>
    );
}
```

Obviously if you have to deal with immutability it is better option passing a function to the updater instead of a direct value.

## useImmerReducer

Immer powered reducer, based on [`useReducer` hook](https://reactjs.org/docs/hooks-reference.html#usereducer)

Example: https://codesandbox.io/s/2zor1monvp

```javascript
import React from 'react';
import { useImmerReducer } from 'use-immer';

const initialState = { count: 0 };

function reducer(draft, action) {
    switch (action.type) {
        case 'reset':
            return initialState;
        case 'increment':
            return void draft.count++;
        case 'decrement':
            return void draft.count--;
    }
}

function Counter() {
    const [state, dispatch] = useImmerReducer(reducer, initialState);
    return (
        <>
            Count: {state.count}
            <button onClick={() => dispatch({ type: 'reset' })}>Reset</button>
            <button onClick={() => dispatch({ type: 'increment' })}>+</button>
            <button onClick={() => dispatch({ type: 'decrement' })}>-</button>
        </>
    );
}
```
